/*
 * Copyright (C) the libgit2 contributors. All rights reserved.
 *
 * This file is part of libgit2, distributed under the GNU GPL v2 with
 * a Linking Exception. For full terms see the included COPYING file.
 */
#ifndef INCLUDE_git_version_h__
#define INCLUDE_git_version_h__

#define LIBGIT2_VERSION "1.1.1"
#define LIBGIT2_VER_MAJOR 1
#define LIBGIT2_VER_MINOR 1
#define LIBGIT2_VER_REVISION 1
#define LIBGIT2_VER_PATCH 0

#define LIBGIT2_SOVERSION "1.1"

#endif
